<?php

/**
 * @author     M2E Pro Developers Team
 * @copyright  M2E LTD
 * @license    Commercial use is forbidden
 */

namespace Ess\M2ePro\Controller\Adminhtml\ControlPanel\Module\Integration;

use Ess\M2ePro\Controller\Adminhtml\Context;
use Ess\M2ePro\Controller\Adminhtml\ControlPanel\Command;
use Ess\M2ePro\Helper\Component\Ebay as EbayHelper;

class Ebay extends Command
{
    /** @var \Ess\M2ePro\Model\ControlPanel\Inspection\Repository */
    protected $repository;
    /** @var \Ess\M2ePro\Model\ControlPanel\Inspection\HandlerFactory */
    protected $handlerFactory;
    /** @var \Ess\M2ePro\Model\ResourceModel\Listing\Product\CollectionFactory */
    private $listingProductCollectionFactory;

    public function __construct(
        \Ess\M2ePro\Helper\View\ControlPanel $controlPanelHelper,
        Context $context,
        \Ess\M2ePro\Model\ControlPanel\Inspection\Repository $repository,
        \Ess\M2ePro\Model\ControlPanel\Inspection\HandlerFactory $handlerFactory,
        \Ess\M2ePro\Model\ResourceModel\Listing\Product\CollectionFactory $listingProductCollectionFactory
    ) {
        parent::__construct($controlPanelHelper, $context);
        $this->repository = $repository;
        $this->handlerFactory = $handlerFactory;
        $this->listingProductCollectionFactory = $listingProductCollectionFactory;
    }

    /**
     * @title "Stop Unmanaged"
     * @description "[in order to resolve the problem with duplicates]"
     * @new_line
     */
    public function stopEbay3rdPartyAction()
    {
        $collection = $this->parentFactory->getObject(EbayHelper::NICK, 'Listing\Other')->getCollection();
        $collection->addFieldToFilter('status', [
            'in' => [
                \Ess\M2ePro\Model\Listing\Product::STATUS_LISTED,
                \Ess\M2ePro\Model\Listing\Product::STATUS_HIDDEN,
            ],
        ]);

        $total = 0;
        $groupedData = [];

        foreach ($collection->getItems() as $item) {
            /** @var \Ess\M2ePro\Model\Ebay\Listing\Other $item */

            $key = $item->getAccount()->getId() . '##' . $item->getMarketplace()->getId();
            $groupedData[$key][$item->getId()] = $item->getItemId();
            $total++;
        }

        foreach ($groupedData as $groupKey => $items) {
            [$accountId, $marketplaceId] = explode('##', $groupKey);

            foreach (array_chunk($items, 10, true) as $itemsPart) {
                /** @var \Ess\M2ePro\Model\Ebay\Connector\Dispatcher $dispatcherObject */
                $dispatcherObject = $this->modelFactory->getObject('Ebay_Connector_Dispatcher');
                $connectorObj = $dispatcherObject->getVirtualConnector(
                    'item',
                    'update',
                    'ends',
                    ['items' => $itemsPart],
                    null,
                    $marketplaceId,
                    $accountId
                );

                $dispatcherObject->process($connectorObj);
                $response = $connectorObj->getResponseData();

                foreach ($response['result'] as $itemId => $iResp) {
                    $item = $this->parentFactory->getObjectLoaded(
                        EbayHelper::NICK,
                        'Listing\Other',
                        $itemId,
                        null,
                        false
                    );
                    if (
                        $item !== null &&
                        ((isset($iResp['already_stop']) && $iResp['already_stop']) ||
                            isset($iResp['ebay_end_date_raw']))
                    ) {
                        $item->setData('status', \Ess\M2ePro\Model\Listing\Product::STATUS_INACTIVE)->save();
                    }
                }
            }
        }

        return "Processed {$total} products.";
    }

    //########################################

    /**
     * @title "Set EPS Images Mode"
     * @description "Set EPS Images Mode = true for listing products"
     * @prompt "Please enter Listing Product ID or `all` code for all products."
     * @prompt_var "listing_product_id"
     */
    public function setEpsImagesModeAction()
    {
        $listingProductId = $this->getRequest()->getParam('listing_product_id');

        $listingProducts = [];
        if (strtolower($listingProductId) == 'all') {
            $listingProducts = $this->parentFactory->getObject(EbayHelper::NICK, 'Listing\Product')->getCollection();
        } else {
            $listingProduct = $this->parentFactory
                ->getObjectLoaded(EbayHelper::NICK, 'Listing\Product', $listingProductId);
            $listingProduct && $listingProducts[] = $listingProduct;
        }

        if (empty($listingProducts)) {
            $this->getMessageManager()->addError('Failed to load Listing Product.');

            return $this->_redirect($this->controlPanelHelper->getPageModuleTabUrl());
        }

        $affected = 0;
        foreach ($listingProducts as $listingProduct) {
            $additionalData = $listingProduct->getAdditionalData();

            if (
                !isset($additionalData['is_eps_ebay_images_mode']) ||
                $additionalData['is_eps_ebay_images_mode'] == true
            ) {
                continue;
            }

            $additionalData['is_eps_ebay_images_mode'] = true;
            $affected++;

            $listingProduct->setData('additional_data', \Ess\M2ePro\Helper\Json::encode($additionalData))
                           ->save();
        }

        $this->getMessageManager()->addSuccess("Set for {$affected} affected Products.");

        return $this->_redirect($this->controlPanelHelper->getPageModuleTabUrl());
    }

    /**
     * @hidden
     */
    public function showNonexistentTemplatesAction()
    {
        $fixData = [
            'field' => $this->getRequest()->getParam('field'),
            'template' => $this->getRequest()->getParam('template'),
            'field_value' => $this->getRequest()->getParam('field_value'),
            'action' => $this->getRequest()->getParam('action'),
            'template_id' => $this->getRequest()->getParam('template_id', false),
        ];

        $definition = $this->repository->getDefinition('NonexistentTemplates');

        /** @var \Ess\M2ePro\Model\ControlPanel\Inspection\Inspector\NonexistentTemplates $inspector */
        $inspector = $this->handlerFactory->create($definition);

        $inspector->fix($fixData);

        $this->_redirect($this->redirect->getRefererUrl());
    }

    /**
     * @title "Show Duplicates [product_id/listing_id]"
     * @description "[MIN(id) will be saved]"
     * @new_line
     */
    public function showEbayDuplicatesAction()
    {
        $structureHelper = $this->getHelper('Module_Database_Structure');

        $listingProduct = $structureHelper->getTableNameWithPrefix('m2epro_listing_product');
        $ebayListingProduct = $structureHelper->getTableNameWithPrefix('m2epro_ebay_listing_product');
        $ebayItem = $structureHelper->getTableNameWithPrefix('m2epro_ebay_item');

        $subQuery = $this->resourceConnection->getConnection()
                                             ->select()
                                             ->from(
                                                 ['melp' => $ebayListingProduct],
                                                 []
                                             )
                                             ->joinInner(
                                                 ['mlp' => $listingProduct],
                                                 'mlp.id = melp.listing_product_id',
                                                 [
                                                     'listing_id',
                                                     'product_id',
                                                     new \Zend_Db_Expr('COUNT(product_id) - 1 AS count_of_duplicates'),
                                                     new \Zend_Db_Expr('MIN(mlp.id) AS save_this_id'),
                                                 ]
                                             )
                                             ->group(['mlp.product_id', 'mlp.listing_id'])
                                             ->having(new \Zend_Db_Expr('count_of_duplicates > 0'));

        $query = $this->resourceConnection->getConnection()
                                          ->select()
                                          ->from(
                                              ['melp' => $ebayListingProduct],
                                              ['listing_product_id', 'ebay_item_id']
                                          )
                                          ->joinInner(
                                              ['mlp' => $listingProduct],
                                              'mlp.id = melp.listing_product_id',
                                              ['status']
                                          )
                                          ->joinInner(
                                              ['templ_table' => $subQuery],
                                              'mlp.product_id = templ_table.product_id AND
                         mlp.listing_id = templ_table.listing_id'
                                          )
                                          ->where('melp.listing_product_id <> templ_table.save_this_id')
                                          ->query();

        $removed = 0;
        $stopped = 0;
        $duplicated = [];

        while ($row = $query->fetch()) {
            if ((bool)$this->getRequest()->getParam('remove', false)) {
                if (
                    $row['status'] == \Ess\M2ePro\Model\Listing\Product::STATUS_LISTED ||
                    $row['status'] == \Ess\M2ePro\Model\Listing\Product::STATUS_HIDDEN
                ) {
                    $dispatcher = $this->modelFactory->getObject('Ebay_Connector_Item_Dispatcher');
                    $dispatcher->process(
                        \Ess\M2ePro\Model\Listing\Product::ACTION_STOP,
                        [$row['listing_product_id']]
                    );

                    $stopped++;
                }

                $this->resourceConnection->getConnection()->delete(
                    $listingProduct,
                    ['id = ?' => $row['listing_product_id']]
                );

                $this->resourceConnection->getConnection()->delete(
                    $ebayListingProduct,
                    ['listing_product_id = ?' => $row['listing_product_id']]
                );

                $this->resourceConnection->getConnection()->delete(
                    $ebayItem,
                    ['id = ?' => $row['ebay_item_id']]
                );

                $removed++;
                continue;
            }

            $duplicated[$row['save_this_id']] = $row;
        }

        if (count($duplicated) <= 0) {
            $message = 'There are no duplicates.';
            $removed > 0 && $message .= ' Removed: ' . $removed;
            $stopped > 0 && $message .= ' Stopped: ' . $stopped;

            return $this->getEmptyResultsHtml($message);
        }

        $tableContent = <<<HTML
<tr>
    <th>Listing ID</th>
    <th>Magento Product ID</th>
    <th>Count Of Copies</th>
</tr>
HTML;
        foreach ($duplicated as $row) {
            $tableContent .= <<<HTML
<tr>
    <td>{$row['listing_id']}</td>
    <td>{$row['product_id']}</td>
    <td>{$row['count_of_duplicates']}</td>
</tr>
HTML;
        }

        $url = $this->getUrl('*/*/*', ['action' => 'showEbayDuplicates', 'remove' => '1']);
        $html = $this->getStyleHtml() . <<<HTML
<html>
    <body>
        <h2 style="margin: 20px 0 0 10px">eBay Duplicates
            <span style="color: #808080; font-size: 15px;">(#count# entries)</span>
        </h2>
        <br/>
        <table class="grid" cellpadding="0" cellspacing="0">
            {$tableContent}
        </table>
        <form action="{$url}" method="get" style="margin-top: 1em;">
            <button type="submit">Remove</button>
        </form>
    </body>
</html>
HTML;

        return str_replace('#count#', count($duplicated), $html);
    }

    //########################################

    /**
     * @title "Try to fix variation product"
     * @description "[]"
     */
    public function tryToFixVariationProductAction()
    {
        if ((bool)$this->getRequest()->getParam('fix', false)) {
            /** @var \Ess\M2ePro\Model\Listing\Product $lp */
            $lpId = $this->getRequest()->getParam('listing_product_id');
            $lp = $this->parentFactory->getObjectLoaded(EbayHelper::NICK, 'Listing\Product', $lpId);

            /** @var \Ess\M2ePro\Model\Ebay\Listing\Product\Variation\Resolver $resolver */
            $resolver = $this->modelFactory->getObject('Ebay_Listing_Product_Variation_Resolver');
            $resolver->setListingProduct($lp);
            $resolver->setIsAllowedToSave((bool)$this->getRequest()->getParam('allowed_to_save'));
            $resolver->setIsAllowedToProcessVariationsWhichAreNotExistInTheModule(true);
            $resolver->setIsAllowedToProcessExistedVariations(true);
            $resolver->setIsAllowedToProcessVariationMpnErrors(true);

            $resolver->resolve();

            $errors = $warnings = $notices = [];
            foreach ($resolver->getMessagesSet()->getEntities() as $message) {
                $message->isError() && $errors[] = $message->getText();
                $message->isWarning() && $warnings[] = $message->getText();
                $message->isNotice() && $notices[] = $message->getText();
            }

            return '<pre>' .
                sprintf('Listing Product ID: %s<br/><br/>', $lpId) .
                sprintf('Errors: %s<br/><br/>', print_r($errors, true)) .
                sprintf('Warnings: %s<br/><br/>', print_r($warnings, true)) .
                sprintf('Notices: %s<br/><br/>', print_r($notices, true)) .
                '</pre>';
        }

        $url = $this->getUrl('*/*/*', ['action' => 'tryToFixVariationProduct']);

        return <<<HTML
<form method="get" enctype="multipart/form-data" action="{$url}">

    <div style="margin: 5px 0; width: 400px;">
        <label style="width: 170px; display: inline-block;">Listing Product ID: </label>
        <input name="listing_product_id" style="width: 200px;" required>
    </div>

    <div style="margin: 5px 0; width: 400px;">
        <label style="width: 170px; display: inline-block;">Allowed to save Item: </label>
        <select name="allowed_to_save" style="width: 200px;" required>
            <option style="display: none;"></option>
            <option value="1">YES</option>
            <option value="0">NO</option>
        </select>
    </div>

    <input name="fix" value="1" type="hidden" />

    <div style="margin: 10px 0; width: 365px; text-align: right;">
        <button type="submit">Show</button>
    </div>

</form>
HTML;
    }

    /**
     * @title "View variations_that_can_not_be_deleted"
     * @description "[View or delete array variations_that_can_not_be_deleted]"
     * @new_line
     */
    public function viewVariationsThatCanNotBeDeletedAction()
    {
        $collection = $this->listingProductCollectionFactory->createWithEbayChildMode();
        $collection->getSelect()->where("additional_data LIKE '%variations_that_can_not_be_deleted%'");
        $collection->getSelect()->where('additional_data NOT LIKE "%\"variations_that_can_not_be_deleted\":[]%"');

        // Delete All action
        if ($this->getRequest()->getParam('delete_all')) {
            foreach ($collection->getItems() as $item) {
                $item->setSetting('additional_data', 'variations_that_can_not_be_deleted', []);
                $item->save();
            }

            $this->getMessageManager()
                 ->addNoticeMessage('Cleared all <code>variations_that_can_not_be_deleted</code> arrays');
            return $this->_redirect($this->redirect->getRefererUrl(), ['_query' => '']);
        }
        // Delete by id action
        $deleteId = $this->getRequest()->getParam('delete_id');
        if (!empty($deleteId)) {
            $collection->addFieldToFilter('id', $deleteId);
            $item = $collection->getFirstItem();
            $item->setSetting('additional_data', 'variations_that_can_not_be_deleted', []);
            $item->save();

            $message = sprintf(
                'Array <code>variations_that_can_not_be_deleted</code> cleared for listing_product_id %s.',
                $deleteId
            );
            $this->getMessageManager()->addNoticeMessage($message);
            return $this->_redirect($this->redirect->getRefererUrl(), ['_query' => '']);
        }

        $html = '';
        foreach ($this->getMessageManager()->getMessages(true)->getItems() as $message) {
            $html .= '<p>' . $message->getText() . '</p>';
        }

        if ($collection->getSize() === 0) {
            $html .= '<p>All products have an empty array <code>variations_that_can_not_be_deleted</code></p>';

            return $this->getResponse()->setBody($html);
        }

        $addTableColumnFunction = function ($content) {
            return '<td style="border: 1px solid black; padding: 3px 5px">' . $content . '</td>';
        };

        $tableHtml = '<div style="padding: 7px 0"><a href="?delete_all=1">Delete All</a></div>';
        $tableHtml .= '<table style="width: 100%; border-collapse: collapse; border: 1px solid black">';
        $tableHtml .= '<tr>'
            . $addTableColumnFunction('listing_product_id')
            . $addTableColumnFunction('product_id')
            . $addTableColumnFunction('variations_that_can_not_be_deleted')
            . $addTableColumnFunction('action')
            . '</tr>';
        /** @var  $item */
        foreach ($collection->getItems() as $item) {
            $additionalData = $item->getSettings('additional_data');
            if (!array_key_exists('variations_that_can_not_be_deleted', $additionalData)) {
                continue;
            }
            $prettyVariations = json_encode($additionalData['variations_that_can_not_be_deleted'], JSON_PRETTY_PRINT);

            $tableHtml .= '<tr>';
            $tableHtml .= $addTableColumnFunction($item->getId());
            $tableHtml .= $addTableColumnFunction($item->getProductId());
            $tableHtml .= $addTableColumnFunction(
                '<textarea rows="5" style="width: 100%">'
                . $prettyVariations
                . '</textarea>'
            );
            $tableHtml .= $addTableColumnFunction("<a href='?delete_id={$item->getId()}'>Delete</a>");
            $tableHtml .= '</tr>';
        }
        $tableHtml .= '</table>';

        return $this->getResponse()->setBody($html . $tableHtml);
    }

    //########################################

    private function getEmptyResultsHtml($messageText)
    {
        $backUrl = $this->controlPanelHelper->getPageModuleTabUrl();

        return <<<HTML
    <h2 style="margin: 20px 0 0 10px">
        {$messageText} <span style="color: grey; font-size: 10px;">
        <a href="{$backUrl}">[back]</a>
    </h2>
HTML;
    }

    //########################################
}
